#include <curses.h>
#include <assert.h>
#include <stdio.h>
#include <string.h>

using namespace std;

#include "config.h"

#include "actor.h"
#include "coord.h"
#include "fov.h"
#include "view.h"
#include "shapes.h"
#include "cave.h"
#include "constants.h"

bool Actor::tryMove(Coord dir, bool force) {
    if( force || cave.canPass(getPos(), dir, false, flying) ) {
	if( isFocus )
	    dirtyFov();
	recompute_fov = true;
	move(dir);
	if( isFocus )
	    dirtyFov(true);
	return true;
    }
    return false;
}

void Actor::dirtyFov(bool setExplored) {
    for ( rectIterator ri(getPos() - Coord(fovDist, fovDist),
		getPos() + Coord(fovDist, fovDist)); ri; ri++ )
	if (inFOV(*ri)) {
	    gameView.setDirty(*ri);
	    if (setExplored)
		cave.setExplored(*ri);
	}
}



bool Actor::inFOV(const Coord& c) {
    const int w = 2*fovDist+1;
    const int h = 2*fovDist+1;
    if ( recompute_fov )
    {
	recompute_fov = false;
	if( fovmap.cells )
	    delete[] fovmap.cells;
	fovmap.width = w;
	fovmap.height = h;
	fovmap.nbcells = w*h;
	fovmap.cells = new cell_t[w*h];
	for( int i=0; i<w*h; i++ ) {
	    TileDef tile = cave.getTileDef(getPos() + Coord(i%w - w/2, i/w - h/2));
	    fovmap.cells[i].fov = 0;
	    fovmap.cells[i].transparent = tile.transparent;
	    fovmap.cells[i].walkable = tile.passable;
	    fovmap.cells[i].cliff_top = tile.cliff == 1;
	    fovmap.cells[i].cliff_foot = tile.cliff == -1;
	}
	TCOD_map_compute_fov_recursive_shadowcasting(
		&fovmap, w/2, h/2, fovDist, true);
    }
    Coord d = c-getPos();
    if( d.sqLen() > fovDist*fovDist )
	return false;
    return( fovmap.cells[ w*(d.y + h/2) + d.x + w/2 ].fov == 1);
}
