#include <assert.h>

using namespace std;

#include "config.h"

#include "basicai.h"
#include "player.h"
#include "event.h"
#include "coord.h"
#include "fov.h"
#include "shapes.h"
#include "cave.h"
#include "glyph.h"
#include "view.h"
#include "messages.h"
#include "random.h"
#include "constants.h"

bool BasicAI::tryHit(Coord dir) {
    if (!cave.canPass(getPos(), dir, true, flying))
	return false;
    Objects os = cave.objectsInCell(getPos()+dir);
    for (Objects::iterator o = os.begin(); o != os.end(); o++)
	if ((*o)->getGlyph().ch == '@') {
	    (*o)->hit(1, "Probe hit!");
	    hit(5);
	    return true;
	}
    return false;
}

ActionReturn BasicAI::interact() {
    Events events;
    events.push_back(new InteractEvent(1, this));

    const int vr = ctp->visionRange;
    const Coord pos = getPos();

    if (offturn) {
	offturn = false;
	return ActionReturn(events);
    }
    else if (ctp->slow)
	offturn = true;

    if (!stunned) {
	if (noInterruptGoal != 0) {
	    if (pos == goal)
		noInterruptGoal = 0;
	    if (noInterruptGoal > 0)
		noInterruptGoal--;
	}
	else if (player.inFOV(pos) && player.hp > 0
		&& (player.getPos() - pos).sqLen() <= vr*vr)
	    goal = player.getPos();

	else if (rani(100) < ctp->wanderChance) {
	    bool goalSet = false;
	    if (rani(100) < ctp->congregateChance) {
		// find a friend...
		vector<Coord> cs;
		for ( rectIterator ri(Coord(-vr, -vr), Coord(vr,vr)); ri; ri++ )
		    if (ri->sqLen() <= vr*vr && *ri != Coord(0,0)) {
			Objects os = cave.objectsInCell(pos + *ri);
			for (Objects::iterator o = os.begin(); o != os.end(); o++)
			    if ((*o)->getGlyph().ch == glyph.ch) {
				// *looks* like a friend!
				cs.push_back(*ri);
			    }
		    }
		if (!cs.empty()) {
		    goalSet = true;
		    goal = pos + cs[ rani(cs.size()) ];
		}
	    }
	    else if (rani(100) < ctp->wallHugChance) {
		vector<Coord> cs;
		for ( rectIterator ri(Coord(-vr, -vr), Coord(vr,vr)); ri; ri++ )
		    if (ri->sqLen() <= vr*vr && cave.cellPassable(*ri)
			    && *ri != Coord(0,0)) {
			for (int d=0; d<8; d++)
			    if (!cave.cellPassable(*ri+dirs[d], true)) {
				cs.push_back(*ri);
				break;
			    }
		    }
		if (!cs.empty()) {
		    goalSet = true;
		    goal = pos + cs[ rani(cs.size()) ];
		}
	    }
	    if (!goalSet) {
		Coord d;
		do d = Coord(rani(2*vr+1)-vr,
			rani(2*vr+1)-vr);
		while (d.sqLen() <= vr*vr && !cave.cellPassable(pos+d));
		goal = pos+d;
	    }
	}

	Coord delta = dirTowards(goal - pos);

	int i;
	for (i=0; i<8; i++)
	    if (dirs[i] == delta)
		break;
	if( i != 8 )
	    for (int r=0; r!=3; r = -2*r+1) {
		Coord dr = dirs[ (i+r) % 8 + (i+r<0 ? 8 :0) ];
		if (tryHit(dr) || tryMove(dr))
		    break;
	}
    }
    else {
	stunned--;
	if (stunned==0) {
	    gameView.setDirty(pos);
	    // post-stun wooziness
	    Coord d;
	    do d = Coord(rani(2*vr+1)-vr,
		    rani(2*vr+1)-vr);
	    while (d.sqLen() <= vr*vr && !cave.cellPassable(pos+d));
	    goal = pos+d;
	    noInterruptGoal = 10;
	}
    }

    return ActionReturn(events);
}

void BasicAI::hit(int damage, const char* message) {
    if (damage > 0 && ( damage > 1 || !ctp->unstunnable )) {
	stunned = max(stunned, 20 + rani(1+2*5)-5);
	gameView.setDirty(getPos());
    }
}

bool BasicAI::isStatic() const {
    return (stunned > 0);
}

const char* BasicAI::describe() const {
    if (stunned)
	return "It's only stunned.";
    else
	return ctp->description;
}

Glyph BasicAI::getGlyph() const {
    Glyph g = glyph;
    if (stunned)
	g.ch = '%';
    else if (offturn)
	return ctp->offturnGlyph;
    return g;
}
