#ifndef INC_EVENT_H
#define INC_EVENT_H

#include <stdio.h>
#include <list>
using namespace std;

#include "object.h"
#include "clock.h"

class Actor;

struct Event;

struct ActionReturn {
    list<Event*> events;

    ActionReturn(list<Event*> events) : events(events) {} 
    ActionReturn() : events() {}
};

struct Event {
    float time;
    virtual ActionReturn execute() { return ActionReturn(); }
    int priority;

    bool operator< (const Event& other) {
	return (time < other.time ||
		(time == other.time && priority > other.priority));
    }

    Event(float time, int priority=0) :
	time(gameClock.time + time), priority(priority) {}

    virtual ~Event() {}
};

struct InteractEvent : public Event {
    Actor* actor;

    ActionReturn execute();
    InteractEvent(float time, Actor* actor, int priority=0) :
	Event(time, priority), actor(actor) {}
};

class EventQueue : private list<Event*> {
    void processActionReturn(ActionReturn ar);
    public:
	void add(Event* event);
	void processNext();
};

extern EventQueue eventQueue;

typedef list<Event*> Events;

#endif /* INC_EVENT_H */
