/*
* libtcod 1.5.0
* Copyright (c) 2008,2009,2010 Jice
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*     * Redistributions of source code must retain the above copyright
*       notice, this list of conditions and the following disclaimer.
*     * Redistributions in binary form must reproduce the above copyright
*       notice, this list of conditions and the following disclaimer in the
*       documentation and/or other materials provided with the distribution.
*     * The name of Jice may not be used to endorse or promote products
*       derived from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY Jice ``AS IS'' AND ANY
* EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
* DISCLAIMED. IN NO EVENT SHALL Jice BE LIABLE FOR ANY
* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
* Modified by Martin Bays 2010 for Threads
* and furtherly 2012 for LagRogue
*/

#include <stdlib.h>
#include <math.h>

#include <fov.h>

#define MAX(a,b) ((a)<(b)?(b):(a))
#define MIN(a,b) ((a)<(b)?(a):(b))

static int mult[4][8]= {
	{1,0,0,-1,-1,0,0,1},
	{0,1,-1,0,0,-1,1,0},
	{0,1,1,0,0,-1,-1,0},
	{1,0,0,1,-1,0,0,-1},
};

enum fovState {
    FOV_CLEAR,
    FOV_BLOCKED,
    FOV_FOOT
};

static void cast_light(map_t *map,int cx, int cy,int row,float start, float end, int radius, int r2,
	int xx, int xy, int yx, int yy, int id, bool light_walls, bool footed) {
    int j;
    if ( start < end ) return;
    fovState state = footed ? FOV_FOOT : FOV_CLEAR;
    for (j=row; j< radius+1; j++) {
	int dx=-j-1;
	int dy=-j;
	float new_start=start;
	while ( dx <= 0 ) {
	    int X,Y;
	    dx++;
	    X=cx+dx*xx+dy*xy;
	    Y=cy+dx*yx+dy*yy;
	    if (!((unsigned)X < (unsigned)map->width &&
			(unsigned)Y < (unsigned)map->height))
		continue;
	    float l_slope,r_slope;
	    cell_t& cell = map->cells[X+Y*map->width];
	    l_slope=(dx-0.5f)/(dy+0.5f);
	    r_slope=(dx+0.5f)/(dy-0.5f);
	    if( start < r_slope ) continue;
	    else if( end > l_slope ) break;

	    fovState newState = FOV_CLEAR;
	    if( !cell.transparent || ( footed && cell.cliff_top ) )
		newState = FOV_BLOCKED;
	    else if( cell.cliff_foot )
		newState = FOV_FOOT;
	    if ( dx*dx+dy*dy <= r2
		    && (light_walls || newState != FOV_BLOCKED)) cell.fov=1;
	    if( state == newState && state != FOV_CLEAR )
		new_start = r_slope;
	    if( state != newState ) {
		if( state != FOV_BLOCKED )
		    cast_light(map,cx,cy,j+1,start,
			    (state==FOV_FOOT && newState==FOV_CLEAR ?
			     new_start : l_slope),
			    radius,r2,
			    xx,xy,yx,yy,id+1,light_walls,(state==FOV_FOOT));
		if( state != FOV_CLEAR )
		    start = new_start;
		if( newState == FOV_FOOT && state != FOV_BLOCKED )
		    start = MIN(start,l_slope);
		if( newState != FOV_CLEAR )
		    new_start = r_slope;
	    }
	    state = newState;
	}
	if ( state == FOV_BLOCKED ) break;
	footed = (state == FOV_FOOT);
    }
}

void TCOD_map_compute_fov_recursive_shadowcasting(map_t* m, int player_x, int player_y, int max_radius, bool light_walls) {
	int oct,r2;
	bool footed;
	if ( max_radius == 0 ) {
		int max_radius_x=m->width-player_x;
		int max_radius_y=m->height-player_y;
		max_radius_x=MAX(max_radius_x,player_x);
		max_radius_y=MAX(max_radius_y,player_y);
		max_radius = (int)(sqrt(max_radius_x*max_radius_x+max_radius_y*max_radius_y))+1;
	}
	r2=max_radius*max_radius;
	footed = m->cells[player_x+player_y*m->width].cliff_foot;
	// recursive shadow casting
	for (oct=0; oct < 8; oct++) cast_light(m,player_x,player_y,1,1.0,0.0,max_radius,r2,
		mult[0][oct],mult[1][oct],mult[2][oct],mult[3][oct],0,light_walls,footed);
	m->cells[player_x+player_y*m->width].fov=1;
}
