#ifndef INC_PLAYER_H
#define INC_PLAYER_H

#include "actor.h"
#include "object.h"
#include "event.h"
#include "fov.h"
#include "constants.h"

struct ActionReturn;

enum OrderType {
    MoveOrder,
    RunOrder,
    FireOrder,
    TestClearOrder,
    AutofireOrder,
    ConsumeFuelOrder,
    ToggleAutofireOrder
};
struct Order {
    OrderType type;
    Coord dir;
    int onFail;
    bool instant;

    Order(OrderType i_type=MoveOrder, Coord i_dir=Coord(0,0), int i_onFail=-1, bool i_instant=false) : 
    type(i_type), dir(i_dir), onFail(i_onFail), instant(i_instant) {}
};
typedef vector<Order> Orders;

class Player;
struct BranchStatus {
    Coord pos;
    int energy, maxenergy;
    int fuel;
    bool autofiring;

    int animTime;

    Coord getPos() { return pos; }
    void update(const Order& order, bool animate=false);
    BranchStatus(Player* pl);
    BranchStatus() {}
};
class Player : public Actor {
    private:
	static const int MAX_BRANCHES = 10;
	Orders branches[MAX_BRANCHES];
	set<int> activeBranches;

	void setBranchStati(bool animate=false);
	void _setBranchStati(const BranchStatus& status, int branch,
	    bool animate=false);
	void dirtyBranch(int branch);
	void dirtyBranches();
	void removeBranch(int branch);
	void moveBranch(int from, int to);

	void examineMode(bool largeScale=false);

	void fire(Coord dir);
	bool autofire();

	void appendOrders();
	Order getOrder(int branch);
	int nextFreeBranch(int branch);
	bool clearPathInDir(Coord dir);
	Coord chooseDirection();

	void updateSeenObjects();

    protected:
	bool tryMove(Coord delta, bool force=false);

    public:
	int hp, maxhp;
	int energy, maxenergy;
	int fuel;
	int lag;
	bool autofiring;

	int selectedBranch;
	BranchStatus branchStati[MAX_BRANCHES];

	int branchPosInCell(Coord pos);
	Glyph branchGlyph(int branch);

	ActionReturn interact();
	void hit(int damage=1, const char* message="The probe is damaged!");

	ObjectMap seenObjects;

	const char* describe() const;

	bool examining;
	Coord examineCell;

	Glyph getGlyph() const;

	void increaseLag();

	bool setGoal(Coord* goal);

	Player(Coord c, Glyph glyph);

	Object* clone() { return new Player(*this); }

    friend class BranchStatus;
};

struct LagIncreaseEvent : public Event {
    ActionReturn execute();
    LagIncreaseEvent(float time, int priority=0) : Event(time, priority) {}
};

extern Player player;

#endif /* INC_PLAYER_H */
