#include <curses.h>
#include <assert.h>

#include "view.h"
#include "coord.h"
#include "cave.h"
#include "object.h"
#include "player.h"
#include "glyph.h"
#include "messages.h"
#include "info.h"

void View::drawAll() {
    for( int i=0; i<w; i++ )
	for( int j=0; j<h; j++ )
	    drawCell(Coord(i,j));
    refresh();
    dirty.clear();
}

void View::drawDirty() {
    for ( set<Coord>::iterator i = dirty.begin(); i != dirty.end(); i++ )
	drawCell(*i);
    refresh();
    dirty.clear();
}

void View::setDirty(Coord c) {
    Coord cc = c - viewTopleft;
    if( 0 <= cc.x && cc.x < w &&
	    0 <= cc.y  && cc.y < h )
	dirty.insert(cc);
}

void View::setFocus(Coord c, bool conservative) {
    Coord cc = c - viewTopleft;
    bool redraw = false;
    if( min(cc.x,w-cc.x) < (conservative ? 2 : max(focusrad,w/4)) ) {
	viewTopleft.x = c.x - w/2;
	redraw = true;
    }
    if( min(cc.y,h-cc.y) < (conservative ? 2 : max(focusrad,h/4)) ) {
	viewTopleft.y = c.y - h/2;
	redraw = true;
    }
    if( redraw )
	drawAll();
}

void View::drawCell(Coord cc) {
    Coord c = cc + viewTopleft;
    drawInCell(getGlyph(c), cc);
}
void View::drawInCell(Glyph glyph, Coord cc) {
    assert( 0 <= cc.x && cc.x < w &&
	    0 <= cc.y  && cc.y < h );
    Coord sc = cc + screenTopleft;
    drawGlyph(sc, glyph);
}

void View::drawAtPos(Glyph glyph, Coord c) {
    Coord cc = c - viewTopleft;
    if( 0 <= cc.x && cc.x < w &&
	    0 <= cc.y  && cc.y < h )
	drawInCell(glyph, cc);
}

void View::refreshView() {
    refresh();
}

Glyph GameView::getGlyph(const Coord& c) {
    Glyph g;

    int b = player.branchPosInCell(c);
    if (b>-1)
	g = player.branchGlyph(b);
    else {
	TileDef t = cave->getTileDef(c);
	g = t.glyph;

	if( t.transparent ) {
	    Objects os = player.seenObjects.objectsInCell(c);
	    for (Objects::iterator o = os.begin(); o != os.end(); o++) {
		g = (*o)->getGlyph();
		if (isObstruction(**o))
		    break;
	    }
	}
	if( !player.inFOV(c) ) {
	    if( cave->isExplored(c) )
		g.col = DARKGREY;
	    else
		g = ' ';
	}
    }

    g.reverse |= (player.examining && player.examineCell == c);

    return g;
}

int firstMessageLine(int h) {
    //return std::max(h-5, std::min(h-1, 25));
    return h-1;
}
void setGeometry() {
    int w,h;
    getmaxyx(stdscr,h,w);
    const int logrollStart = firstMessageLine(h);
    erase();
    gameView.setGeom(w-12, logrollStart-2, Coord(1,1));
    const int infoHeight = std::max(5, logrollStart/2);
    infoPanel.setGeom(Coord(w-10,0),10,infoHeight);
    helpPanel.setGeom(Coord(w-10,infoHeight),10,logrollStart-infoHeight);
    logroll.setGeom(logrollStart,w,h-logrollStart);
    indicators.setGeom(Coord(0,0), Coord(w-11,logrollStart-1));
    refresh();
}
